"""
$RCSfile: CVSSandboxRegistry.py,v $

The CVSFile package is a Zope product.  CVSFile allows one to use CVS
as a version control system for Zope content, by pointing to a file in
the developer's CVS sandbox on the Zope server.  CVSSandboxRegistry
encapsulates information about CVS sandboxes, which is obtained by
CVSFile objects via acquisition.

Author: <a href="mailto:cstrong@arielpartners.com">Craeg Strong</a>
Release: 1.0
"""

__cvstag__  = '$Name:  $'[6:-2]
__date__    = '$Date: 2003/04/21 04:23:24 $'[6:-2]
__version__ = '$Revision: 1.13 $'[10:-2]

# Python builtins
import os.path

# Zope builtins
from Globals import MessageDialog
from AccessControl import ClassSecurityInfo
from Acquisition import aq_parent
import Globals  # InitializeClass (security stuff)
import OFS                   # we grab his _property settings
from DateTime.DateTime import DateTime
from webdav.common import rfc1123_date
from Products.PageTemplates.PageTemplateFile import PageTemplateFile

# Add our base classes.  Derived classes will get this stuff automatically
from OFS.SimpleItem import SimpleItem  # The SimpleItem module is a Zope builtin that provides
                                       # cut/paste, management views, webdav/ftp, undo, ownership,
                                       # traversal, acquisition, and persistence
from Products.ZCatalog.CatalogAwareness import CatalogAware # Reqd for ZCatalog support
from OFS.PropertyManager import PropertyManager  # Reqd for proper property mgmt
from SandboxInfo import SandboxInfo
from ISandboxRegistry import ISandboxRegistry
from OSUtils import runCommand, command_separator, canonicalizeDir

################################################################
# Permissions
################################################################
# By defining these constants, no new permissions will be created when
# misspelling them in the declareProtected() call

PERM_VIEW  = "View"
PERM_EDIT  = "Manage CVS Sandboxes"

################################################################
# Container Methods
################################################################

def addInstance(folder, id, title='', description='',
                sandboxname='', sandboxdir='',cvsroot='',
                REQUEST=None, RESPONSE=None):
    """
    This is a convenience factory method for creating an instance of
    CVSSandboxRegistry.  It returns the object created, and may
    therefore be more convenient than the manage_add() method it
    calls.  It is used by the unit testing programs.
    """
    folder.manage_addProduct['CVSFile'].addCVSSandboxRegistry(
        id, title, description, sandboxname,
        sandboxdir, cvsroot)
    return folder[id]

################################################################
# Constructors
################################################################

# globals() is standard python for passing the current namespace
manage_addCVSSandboxRegistryForm = PageTemplateFile('www/create_registry.pt', globals())

def findCVSSandboxRegistry(self):
    """

    Find an instance of CVSSandboxRegistry to obtain sandbox
    information.
    
    All CVSFile instances require an instance of CVSSandboxRegistry.
    The current policy is as follows: use the first registry found
    by acquisition.  Alternatively, a more complex policy could be
    used such as 
    a) looking in some predefined place, where the place could be 
    specified in some property (a la Zope3 explicit acquisition), 
    or 
    b) finding all registries in the ZODB and presenting a list for 
    selection by the user.
    For right now, we don't want to over-engineer the damn thing :)
    CKS 2/14/2002.
      
    """
    metaType = 'CVS Sandbox Registry'
    
    reggie = None
    folder = self
    root   = self.getPhysicalRoot()

    while reggie is None:
        if folder.isPrincipiaFolderish:
            registries = folder.objectValues(metaType)
            if registries:
                reggie = registries[0]
        if reggie is None:
            if folder is root:
                return None
            else:
                folder = aq_parent(folder)
    return reggie

    # Below gets confused by context aquisition.  Bad idea.
    #
    # Note: superValues does _NOT_ find things in the "Root Folder"
    #registries = self.superValues('CVS Sandbox Registry')
    #if registries:
    #    return registries[0]
    #else:
    #    return None

def manage_addCVSSandboxRegistry(self,
                                 id='', title='', description='',          # standard
                                 sandboxname='',sandboxdir='',cvsroot='',  # cvs registry
                                 REQUEST=None, RESPONSE=None):
    """
    Add a CVSSandboxRegistry to a folder.  Called from the
    create_registry form.  It calls addCVSSandboxRegistry to actually
    do the work.
    """
    try:
        self.addCVSSandboxRegistry(id, title, description,
                                   sandboxname, sandboxdir, cvsroot)
        if REQUEST is None:
            return

        # support Add and Edit
        try:
            url = self.DestinationURL()
        except:
            url = REQUEST['URL1']
        if REQUEST['action'].find("Edit") != -1:
            url = "%s/%s" % (url, id)
            REQUEST.RESPONSE.redirect(url + '/manage_registryForm')
        else:
            REQUEST.RESPONSE.redirect(url + "/manage_main")
        
    except Exception, e:
        message = str(e)
        message.replace('\n','<br/>')
        return MessageDialog(title   = 'Error',
                             message = message,
                             action  = 'manage_main')
        

def addCVSSandboxRegistry(self,
                          id='', title='', description='',           # standard
                          sandboxname='',sandboxdir='',cvsroot=''):  # cvs registry
    """
    Factory method to actually create a CVSSandboxRegistry.  Call this
    method directly if creating an instance of CVSSandboxRegistry
    programmatically.
    """
    if not id or not sandboxname or not sandboxdir or not cvsroot:
        raise Exception('Required fields must not be blank')

    if not os.path.exists(sandboxdir) or not os.path.isdir(sandboxdir):
        message = '%s is not a valid directory' % (sandboxdir)
        raise Exception(message)

    self._setObject(id, CVSSandboxRegistry(id, title,
                                           description,
                                           SandboxInfo(sandboxname,
                                                       canonicalizeDir(sandboxdir),
                                                       cvsroot) ))
    self._getOb(id).reindex_object()

################################################################
# CVSSandboxRegistry class
################################################################

# By default when doing a recursive checkout of the
# entire sandbox, checkout this module
default_cvs_module      = 'com'

# By default, whenever doing a CVS command, specify this argument
# -P is "prune empty directories"
default_cvs_args        = '-P'

# default cookie name
default_cookie_name     = 'CVSSandbox'

class CVSSandboxRegistry(CatalogAware,    # ZCatalog support
                         PropertyManager, # Property support
                         SimpleItem):

    """
    CVSSandboxRegistry
    """

    meta_type = 'CVS Sandbox Registry'  # This is the name Zope will use for the
    # Product in the "addProduct" list

    __implements__ = ISandboxRegistry
    
    # This tuple defines our properties
    _properties = (
        {'id':'title',                               'type':'string',    'mode': 'w'},
        {'id':'description',                         'type':'string',    'mode': 'w'},
        {'id':'default_cvs_module_to_checkout',      'type':'string',    'mode': 'w'},
        {'id':'default_cvs_command_line_parameters', 'type':'string',    'mode': 'w'},        
        )

    # This tuple defines a dictionary for each tab in the management interface
    # label = label of tab, action = url it links to
    manage_options = (
        {'label':'Registry', 'action': 'manage_registryForm', 'help': ('CVSFile', 'registry.stx') },
        {'label':'Sandbox',  'action': 'manage_sandboxForm',  'help': ('CVSFile', 'sandbox.stx') },
        ) + OFS.PropertyManager.PropertyManager.manage_options + OFS.SimpleItem.SimpleItem.manage_options

    _security = ClassSecurityInfo()

    # set security for the object itself, e.g. if it is accessed in DTML code
    # This line is REQUIRED to allow access to CVSFiles by the public
    _security.declareObjectPublic()

    _security.declareProtected(PERM_EDIT, 'manage_sandboxForm')
    manage_sandboxForm = PageTemplateFile('www/sandbox.pt',globals())

    _security.declareProtected(PERM_EDIT, 'manage_registryForm')
    manage_registryForm = PageTemplateFile('www/registry.pt',globals())

    def __init__(self, id, title, description, default_sandbox):
        """
        Initialize a new instance of CVSSandboxRegistry
        """
        self.id             = id       # This is the OID translated by Zope into a URL
        self.title          = title
        self.description    = description
        self.__version__    = __version__
        self.sandboxes      = { default_sandbox.name : default_sandbox }
        self.defaultSandbox = default_sandbox
        
        self.default_cvs_module_to_checkout      = default_cvs_module
        self.default_cvs_command_line_parameters = default_cvs_args
        self.cookie_name                         = default_cookie_name

    ################################################################
    # Methods implementing the ISandboxRegistry interface
    ################################################################

    def sandboxList(self):
        "Returns the list of sandboxes"
        return self.sandboxes.values()
        
    def numSandboxes(self):
        "return the number of sandboxes currently registered"
        return len(self.sandboxes)

    def getSandboxByName(self, sandboxName):
        "return the named sandbox, or None"
        return self.sandboxes.get(sandboxName, None)

    _security.declareProtected(PERM_VIEW,  'isSelected')
    def isSelected(self, sandboxName, REQUEST):
        """
        
        Return 1 if name represents the currently selected sandbox,
        or 0 if it is not, or if the name does not exist

        """
        return REQUEST and REQUEST.has_key(self.cookie_name) and REQUEST[self.cookie_name] == sandboxName

    _security.declareProtected(PERM_VIEW,  'isDefault')
    def isDefault(self, sandboxName):
        """
        
        Return 1 if name represents the default sandbox,
        or 0 if it is not, or if the name does not exist

        """
        return self.defaultSandbox.name == sandboxName

    _security.declareProtected(PERM_VIEW,  'getSelectedSandbox')
    def getSelectedSandbox(self, REQUEST):
        "return the selected sandbox or None"
        if REQUEST and REQUEST.has_key(self.cookie_name):
            return self.getSandboxByName(REQUEST[self.cookie_name])
        else:
            return None

    _security.declareProtected(PERM_VIEW,  'getDefaultSandbox')
    def getDefaultSandbox(self):
        "return the default sandbox"
        return self.defaultSandbox

    _security.declareProtected(PERM_VIEW,  'getCurrentSandbox')
    def getCurrentSandbox(self, REQUEST):
        """
        return the selected sandbox or the default, if none has been
        selected
        """
        si = self.getSelectedSandbox(REQUEST)
        if si:
            return si
        else:
            return self.getDefaultSandbox()

    _security.declareProtected(PERM_EDIT, 'setDefaultSandbox')
    def setDefaultSandbox(self, sandboxName):
        """

        set the default sandbox to the one whose name is passed.  Barf
        if the name does not exist in this registry

        """
        if not self.sandboxes.has_key(sandboxName):
            message = 'No sandbox named ' + sandboxName + ' exists in this registry.'
            raise ValueError, message
        si = self.sandboxes[sandboxName]
        self.defaultSandbox = si
        self._p_changed = 1  # dirty flag, triggers persistence to update the object        

    _security.declareProtected(PERM_EDIT, 'selectSandbox')
    def selectSandbox(self, sandboxName, REQUEST):
        """
        Select the named sandbox.  Barf if the named sandbox does not
        exist.
        """
        if not self.sandboxes.has_key(sandboxName):
            message = 'No sandbox named ' + sandboxName + ' exists in this registry.'
            raise ValueError, message
        if REQUEST:
            e=(DateTime('GMT') + 365).rfc822()
            resp = REQUEST['RESPONSE']
            #
            # path   = "/" means apply to all pages within this website
            # expire = "+1yr means cookie stays around for a year"
            #
            # see www.netscape.com/newsref/std/cookie_spec.html
            #
            resp.setCookie(self.cookie_name, sandboxName,
                           path='/', expires=e)

    _security.declareProtected(PERM_EDIT, 'addSandbox')
    def addSandbox(self, si):
        """
        Add a sandbox to the registry.  Barf if it has the same name
        as an existing sandbox
        """
        if self.sandboxes.has_key(si.name):
            message = 'A sandbox named ' + si.name + ' already exists in this registry.'
            raise ValueError, message
        self.sandboxes[si.name] = si
        self._p_changed = 1  # dirty flag, triggers persistence to update the object        

    _security.declareProtected(PERM_EDIT, 'removeSandbox')
    def removeSandbox(self, sandboxName):
        """
        Remove a sandbox from the registry.  Barf if it does not exist
        or is the default sandbox
        """
        if not self.sandboxes.has_key(sandboxName):
            message = 'No sandbox named ' + sandboxName + ' exists in this registry.'
            raise ValueError, message

        if self.defaultSandbox.name == sandboxName:
            message = 'The default sandbox can not be removed.'
            raise ValueError, message

        del self.sandboxes[sandboxName]
        self._p_changed = 1  # dirty flag, triggers persistence to update the object        

    _security.declareProtected(PERM_EDIT, 'changeSandbox')
    def changeSandbox(self, nameOfSandboxToChange, newSI):
        """
        Change the named sandbox to have the new name, dir, and
        cvsroot values.
        """
        if not self.sandboxes.has_key(nameOfSandboxToChange):
            message = 'No sandbox named ' + nameOfSandboxToChange + ' exists in this registry.'
            raise ValueError, message

        if self.sandboxes.has_key(newSI.name):
            message = 'A sandbox named ' + newSI.name + ' already exists in this registry.'
            raise ValueError, message
            
        si = self.sandboxes[nameOfSandboxToChange]
        si.name    = newSI.name
        si.basedir = newSI.basedir
        si.cvsroot = newSI.cvsroot
        del self.sandboxes[nameOfSandboxToChange]
        self.sandboxes[newSI.name] = si
        self._p_changed = 1  # dirty flag, triggers persistence to update the object

    _security.declareProtected(PERM_EDIT, 'changeSandbox')
    def checkoutSandbox(self, checkoutModule, checkoutArgs, si):
        """
        Does a CVS checkout recursively on the entire sandbox, using
        the sandbox information from the SandboxInfo parameter
        """
        command = "cd %s %s cvs -d %s -q checkout %s %s" % ( si.basedir,
                                                             command_separator,
                                                             si.cvsroot,
                                                             checkoutArgs,
                                                             checkoutModule )
        output = runCommand(command)
        return command + "\n" + output

    ################################################################
    # Methods called from sandbox form
    ################################################################

    # methods called from forms
    _security.declareProtected(PERM_EDIT,  'manage_addSandbox')
    def manage_addSandbox(self,
                          name='', basedir='', cvsroot='',
                          REQUEST=None, RESPONSE=None):
        "Add a new sandbox to the registry.  Called from registry GUI form."

        message = None
        if not name or not basedir or not cvsroot:
            message = 'All values must be filled in'
        elif not os.path.exists(basedir) or not os.path.isdir(basedir):
            message = basedir + ' is not a valid SANDBOXDIR'
            
        if message:
            if REQUEST:
                return MessageDialog(title   = 'ERROR',
                                     message = message,
                                     action  = 'manage_registryForm')
            else:
                raise ValueError, message
        try:
            si = SandboxInfo(name, canonicalizeDir(basedir), cvsroot)
            self.addSandbox(si)
        except ValueError, message:
            return MessageDialog(title   = 'ERROR',
                                 message = message,
                                 action  = 'manage_registryForm')
        return self.manage_registryForm()

    _security.declareProtected(PERM_EDIT,  'manage_selectSandbox')
    def manage_selectSandbox(self, sandboxName=None,
                             REQUEST={}, RESPONSE=None):
        """
        Select a sandbox to the registry.  Called from registry GUI
        form.  More than one cannot be selected, b/c we are using a
        radio box for selection.
        """
        if not sandboxName:
            return MessageDialog(title   = 'ERROR',
                                 message = 'You must first select a sandbox.',
                                 action  = 'manage_registryForm')
        try:
            self.selectSandbox(sandboxName, REQUEST)
        except ValueError, message:
            return MessageDialog(title   = 'ERROR',
                                 message = message,
                                 action  = 'manage_registryForm')
        return self.manage_registryForm(selectedSandbox = sandboxName)

    _security.declareProtected(PERM_EDIT,  'manage_removeSandbox')
    def manage_removeSandbox(self, sandboxName=None,
                             REQUEST=None, RESPONSE=None):
        """
        Remove a sandbox from the registry.  Called from registry GUI
        form.  More than one cannot be removed at one time, b/c we are
        using a radio box for selection.
        """
        if not sandboxName:
            return MessageDialog(title   = 'ERROR',
                                 message = 'You must first select a sandbox.',
                                 action  = 'manage_registryForm')
        try:
            self.removeSandbox(sandboxName)
        except ValueError, message:
            return MessageDialog(title   = 'ERROR',
                                 message = message,
                                 action  = 'manage_registryForm')
        return self.manage_registryForm()

    _security.declareProtected(PERM_EDIT,  'manage_setDefaultSandbox')
    def manage_setDefaultSandbox(self, sandboxName=None,
                                 REQUEST=None, RESPONSE=None):
        """
        Designate a particular sandbox in the registry as the default.
        Called from registry GUI form.  More than one cannot be
        selected, b/c we are using a radio box for selection.
        """
        if not sandboxName:
            return MessageDialog(title   = 'ERROR',
                                 message = 'You must first select a sandbox.',
                                 action  = 'manage_registryForm')
        try:
            self.setDefaultSandbox(sandboxName)
        except ValueError, message:
            return MessageDialog(title   = 'ERROR',
                                 message = message,
                                 action  = 'manage_registryForm')
        return self.manage_registryForm(defaultSandbox = sandboxName)

    _security.declareProtected(PERM_EDIT,  'manage_setDefaultSandbox')
    def manage_changeSandbox(self, sandboxName,
                             REQUEST=None, RESPONSE=None):
        """
        Change the named sandbox to have the new name, dir, and
        cvsroot values.
        """
        return MessageDialog(title   = 'ERROR',
                             message = 'Not implemented yet',
                             action  = 'manage_registryForm')

    # called from sandbox GUI form
    _security.declareProtected(PERM_EDIT,  'cvsCheckoutSandbox')
    def manage_CheckoutSandbox(self, checkoutModule='', checkoutArgs='',
                               REQUEST={}):
        """
        Does a CVS checkout recursively on the entire sandbox and
        passes output back to the form
        """
        si     = self.getCurrentSandbox(REQUEST)
        output = self.checkoutSandbox(checkoutModule, checkoutArgs, si)
        return self.manage_sandboxForm(checkoutSandboxOutput = output)

    ################################################################
    # Standard Zope stuff
    ################################################################

    _security.declareProtected(PERM_VIEW,  '__str__')
    def __str__(self):
        message = self.id + " (" + self.title + ")"
        return message

    ################################################################
    # Support for Schema Migration
    ################################################################

    def repair(self):
        """
        Repair this object.  This method is used for schema migration,
        when the class definition changes and existing instances of
        previous versions must be updated.
        """
        self.cookie_name = default_cookie_name
        
# register security information
Globals.InitializeClass(CVSSandboxRegistry)
