"""
$RCSfile: TestCVSSandboxRegistry.py,v $
Unit tests for CVSSandboxRegistry object within the CVSFile product

Author: <a href="mailto:cstrong@arielpartners.com">Craeg Strong</a>
Release: 1.0
"""

__cvstag__  = '$Name:  $'[6:-2]
__date__    = '$Date: 2003/04/19 05:08:35 $'[6:-2]
__version__ = '$Revision: 1.4 $'[10:-2]

import os, os.path, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

# Load fixture
from Testing import ZopeTestCase

from Products.CVSFile import CVSSandboxRegistry
from Products.CVSFile.SandboxInfo import SandboxInfo
from OFS.ObjectManager import BadRequestException

ZopeTestCase.installProduct('ExternalFile')
ZopeTestCase.installProduct('CVSFile')

class CVSSandboxRegistryTestCase(ZopeTestCase.ZopeTestCase):

    def afterSetUp(self):
        'Add object to default fixture'
        self.boxname = 'cks'
        self.basedir = os.path.normpath('/home/cstrong/ade')
        self.cvsroot = '/repository/cvsroot'

        self._reg = CVSSandboxRegistry.addInstance(self.folder, 'qa', '', '',
                                                   self.boxname,
                                                   self.basedir,
                                                   self.cvsroot)

    def test_01(self):
        "Ensure sandbox registry can be created with one sandbox"

        assert self._reg.numSandboxes() == 1

        si = self._reg.defaultSandbox
        self.assertEquals(si.name,    self.boxname)
        self.assertEquals(os.path.normpath(si.basedir), os.path.normpath(self.basedir))
        self.assertEquals(si.cvsroot, self.cvsroot)

        si2 = self._reg.getSandboxByName(self.boxname)
        self.assertEquals(si,si2)

        si3 = self._reg.getDefaultSandbox()
        self.assertEquals(si,si3)

        si4 = self._reg.getCurrentSandbox(self.app.REQUEST)
        self.assertEquals(si,si4)

        self.failUnless(self._reg.isDefault(self.boxname))
        self.failIf(self._reg.getSelectedSandbox(self.app.REQUEST))

        self._reg.selectSandbox(self.boxname, self.app.REQUEST)

        #si5 = self._reg.getCurrentSandbox(self.boxname, self.app.REQUEST)
        #self.assertEquals(si,si5)

    def test_02(self):
        "Ensure registry does not allow duplicate sandboxes"

        self.assertRaises(BadRequestException, CVSSandboxRegistry.addInstance,
                          self.folder, 'qa', '', '',
                          self.boxname, self.basedir, self.cvsroot)

    def test_03(self):
        "Ensure that multiple sandboxes can be added to a registry"

        si = SandboxInfo('WGM dev', os.path.normpath('/home/cmorris/ade'),
                         ':pserver:cstrong@cvs.arielpartners.com:/repository/cvsroot') 
        self._reg.addSandbox(si)

        si3 = self._reg.getDefaultSandbox()
        self.failIf(si == si3)
        self.assertEquals(self._reg.numSandboxes(), 2)
        list = self._reg.sandboxList()
        assert len(list) == 2

        self._reg.setDefaultSandbox("WGM dev")

        si4 = self._reg.getSandboxByName("WGM dev")
        si5 = self._reg.getDefaultSandbox()
        self.assertEquals(si4, si5)
        self.failUnless(self._reg.isDefault("WGM dev"))

    def test_04(self):
        "Try to set the default sandbox to one that does not exist"

        self.assertRaises(ValueError, self._reg.setDefaultSandbox,
                          "i don't exist")

    def test_05(self):
        "Ensure that a sandbox can be removed"
        
        self.assertEquals(self._reg.numSandboxes(),1)
        self._reg.addSandbox(SandboxInfo("WGM dev", "/home/cmorris/ade",
                                         ":pserver:cstrong@cvs.arielpartners.com:/repository/cvsroot"))
        self.assertEquals(self._reg.numSandboxes(),2)
        self._reg.addSandbox(SandboxInfo("WGM dev2", "/home/cmorris/ade",
                                         ":pserver:cstrong@cvs.arielpartners.com:/repository/cvsroot"))
        self.assertEquals(self._reg.numSandboxes(),3)

        self._reg.removeSandbox("WGM dev")
        self.assertEquals(self._reg.numSandboxes(),2)
        
    def test_06(self):
        "Attempt to remove nonexistent sandbox should fail"
        
        self.assertEquals(self._reg.numSandboxes(),1)
        self._reg.addSandbox(SandboxInfo("WGM dev", "/home/cmorris/ade",
                                         ":pserver:cstrong@cvs.arielpartners.com:/repository/cvsroot"))
        self.assertEquals(self._reg.numSandboxes(),2)
        self._reg.addSandbox(SandboxInfo("WGM dev2", "/home/cmorris/ade",
                                         ":pserver:cstrong@cvs.arielpartners.com:/repository/cvsroot"))
        self.assertEquals(self._reg.numSandboxes(),3)

        self.assertRaises(ValueError, self._reg.removeSandbox,
                          "i don't exist")
        self.assertEquals(self._reg.numSandboxes(),3)

    def test_07(self):
        "Attempt to remove the default sandbox should fail"
        
        self.assertEquals(self._reg.numSandboxes(),1)
        self._reg.addSandbox(SandboxInfo("WGM dev", "/home/cmorris/ade",
                                         ":pserver:cstrong@cvs.arielpartners.com:/repository/cvsroot"))
        self.assertEquals(self._reg.numSandboxes(),2)
        self._reg.addSandbox(SandboxInfo("WGM dev2", "/home/cmorris/ade",
                                         ":pserver:cstrong@cvs.arielpartners.com:/repository/cvsroot"))
        self.assertEquals(self._reg.numSandboxes(),3)

        self._reg.setDefaultSandbox("WGM dev")

        self._reg.removeSandbox(self.boxname)
        self.assertRaises(ValueError, self._reg.removeSandbox,
                          'WGM dev')
        self.assertEquals(self._reg.numSandboxes(),2)

    def test_08(self):
        "Attempt to change a nonexistent sandbox should fail"

        self.assertRaises(ValueError, self._reg.changeSandbox,
                          'WGM dev', SandboxInfo('foo', 'bar', 'mumble'))
        
    def test_09(self):
        "Attempt to change a sandbox"
        
        self._reg.addSandbox(SandboxInfo("WGM dev", "/home/cmorris/ade",
                                         ":pserver:cstrong@cvs.arielpartners.com:/repository/cvsroot"))

        self._reg.changeSandbox("WGM dev",
                                SandboxInfo("foo", "bar", "mumble"))

        self.failIf(self._reg.getSandboxByName("WGM dev"))
        si = self._reg.getSandboxByName("foo")
        self.assertEquals(si.name,    'foo')
        self.assertEquals(si.basedir, 'bar')
        self.assertEquals(si.cvsroot, 'mumble')
        assert si.name    == "foo"
        assert si.basedir == "bar"
        assert si.cvsroot == "mumble"

    def test_10(self):
        "Attempt to change a sandbox to a name that already exists should fail"
        self._reg.addSandbox(SandboxInfo("WGM dev", "/home/cmorris/ade",
                                         ":pserver:cstrong@cvs.arielpartners.com:/repository/cvsroot"))

        self.assertRaises(ValueError, self._reg.changeSandbox,
                          'WGM dev',SandboxInfo(self.boxname, "bar", "mumble")) 
        
# How to test selection?
# create registry, ask which one is selected (none)
# add another one
# select the first, verify that it is selected
# make the second one the default
# delete the first one
# ask which one is selected (none)
# select the (now) first one, verify its selected
# create eight more, now select the last one, verify its selected

    ################################################################
    # Test Runner Setup
    ################################################################

if __name__ == '__main__':
    framework(descriptions=1, verbosity=2) # 0=quiet 1=default 2=verbose
else:
    # While framework.py provides its own test_suite() 
    # method the testrunner utility does not.
    import unittest
    def test_suite():
        suite = unittest.TestSuite()
        suite.addTest(unittest.makeSuite(CVSSandboxRegistryTestCase))
        return suite
